@@ -1,4 +1,7 @@
 package HTML::FormFu::Constraint;
+{
+  $HTML::FormFu::Constraint::VERSION = '1.00';
+}
 
 use strict;
 use base 'HTML::FormFu::Processor';
@@ -9,6 +12,7 @@ extends 'HTML::FormFu::Processor';
 use HTML::FormFu::Exception::Constraint;
 use HTML::FormFu::Util qw(
     DEBUG_CONSTRAINTS
+    DEBUG_CONSTRAINTS_WHEN
     debug
 );
 use Clone ();
@@ -20,19 +24,55 @@ use List::MoreUtils qw( any all );
 use List::Util qw( first );
 use Scalar::Util qw( reftype blessed );
 
-has not          => ( is => 'rw', traits  => ['Chained'] );
-has force_errors => ( is => 'rw', traits  => ['Chained'] );
-has when         => ( is => 'rw', traits  => ['Chained'] );
-has only_on_reps => ( is => 'rw', traits  => ['Chained'] );
+has not          => ( is => 'rw', traits => ['Chained'] );
+has force_errors => ( is => 'rw', traits => ['Chained'] );
+has when         => ( is => 'rw', traits => ['Chained'] );
+has only_on_reps => ( is => 'rw', traits => ['Chained'] );
 
-sub pre_process {}
+sub repeatable_repeat {
+    my ( $self, $repeatable, $new_block ) = @_;
+
+    my $block_fields = $new_block->get_fields;
+
+    # rename any 'when' fields
+    {
+        my $when = $self->when;
+
+        if ( my $name = $when->{field} ) {
+            my $field = $repeatable->get_field_with_original_name( $name,
+                $block_fields );
+
+            if ( defined $field ) {
+                DEBUG_CONSTRAINTS && debug(
+                    sprintf
+                        "Repeatable renaming constraint 'when{field}' '%s' to '%s'",
+                    $name, $field->nested_name,
+                );
+
+                $when->{field} = $field->nested_name;
+            }
+        }
+        elsif ( my $names = $when->{fields} ) {
+            for my $name (@$names) {
+                my $field = $repeatable->get_field_with_original_name( $name,
+                    $block_fields );
+
+                if ( defined $field ) {
+                    $when->{field} = $field->nested_name;
+                }
+            }
+        }
+    }
+}
+
+sub pre_process { }
 
 sub process {
     my ( $self, $params ) = @_;
 
     return unless $self->_run_this_rep;
 
-    my $value = $self->_find_field_value( $params );
+    my $value = $self->_find_field_value($params);
 
     my @errors;
 
@@ -71,16 +111,16 @@ sub process {
 
 sub _run_this_rep {
     my ($self) = @_;
-    
+
     my $only_on_reps = $self->only_on_reps
         or return 1;
-    
+
     my $current_rep = $self->field->repeatable_count
         or return 1;
-    
+
     $only_on_reps = [$only_on_reps]
         if ( reftype($only_on_reps) || '' ) ne 'ARRAY';
-    
+
     return first { $current_rep == $_ } @$only_on_reps;
 }
 
@@ -89,37 +129,38 @@ sub _find_field_value {
 
     my $value = $self->get_nested_hash_value( $params, $self->nested_name );
 
-    my @fields_with_this_name = @{ $self->form->get_fields({ nested_name => $self->nested_name }) };
-    
+    my @fields_with_this_name
+        = @{ $self->form->get_fields( { nested_name => $self->nested_name } ) };
+
     if ( @fields_with_this_name > 1 ) {
         my $field = $self->parent;
         my $index;
-        
-        for ( my $i=0; $i <= $#fields_with_this_name; ++$i ) {
+
+        for ( my $i = 0; $i <= $#fields_with_this_name; ++$i ) {
             if ( $fields_with_this_name[$i] eq $field ) {
                 $index = $i;
                 last;
             }
         }
-        
+
         croak 'did not find ourself - how can this happen?'
             if !defined $index;
-        
-        if ( reftype($value) eq 'ARRAY' ) {
+
+        if ( ( reftype($value) || '' ) eq 'ARRAY' ) {
             $value = $value->[$index];
         }
         elsif ( $index == 0 ) {
+
             # keep $value
         }
         else {
             undef $value;
         }
     }
-    
+
     return $value;
 }
 
-
 sub constrain_values {
     my ( $self, $values, $params ) = @_;
 
@@ -195,49 +236,53 @@ sub _process_when {
     my $when_fields    = $when->{fields};
     my $when_any_field = $when->{any_field};
     my $when_callback  = $when->{callback};
-    
-    croak "'field', 'fields', 'any_field' or 'callback' key must be defined in 'when'"
-        if all {!defined} $when_field, $when_fields, $when_any_field, $when_callback;
+
+    croak
+        "'field', 'fields', 'any_field' or 'callback' key must be defined in 'when'"
+        if all { !defined } $when_field, $when_fields, $when_any_field,
+        $when_callback;
 
     # Callback will be the preferred thing
     if ($when_callback) {
         no strict 'refs';
-        return $when_callback->($params);
+        return $when_callback->( $params, $self );
     }
-    
+
     my $any;
     my @when_fields_value;
-    
+
     if ($when_any_field) {
         croak "'any_field' is set to an empty list" if !@$when_any_field;
 
         $any = 1;
-        
+
         @$when_fields = @$when_any_field;
     }
-    
+
     if ($when_fields) {
         croak "'fields' is set to an empty list" if !@$when_fields;
-        
+
         for my $name (@$when_fields) {
             my $value = $self->get_nested_hash_value( $params, $name );
-            
+
             push @when_fields_value, $value
                 if defined $value;
         }
     }
     else {
+
         # nothing to constrain if field doesn't exist
         my $value = $self->get_nested_hash_value( $params, $when_field );
-        
+
         push @when_fields_value, $value
             if defined $value;
     }
 
-    DEBUG_CONSTRAINTS && debug('WHEN_FIELDS_VALUES' => \@when_fields_value);
+    DEBUG_CONSTRAINTS_WHEN && debug( 'WHEN_FIELDS_VALUES' => \@when_fields_value );
 
-    if (!@when_fields_value) {
-        DEBUG_CONSTRAINTS && debug("No 'when' fields values exist - returning false");
+    if ( !@when_fields_value ) {
+        DEBUG_CONSTRAINTS_WHEN
+            && debug("No 'when' fields values exist - returning false");
         return 0;
     }
 
@@ -263,18 +308,31 @@ sub _process_when {
             push @ok, $value ? 1 : 0;
         }
     }
-    
-    DEBUG_CONSTRAINTS && debug("'when' value matches" => \@ok);
 
-    my $return = $any ? any { $when->{not} ? !$_ : $_ } @ok
-               :        all { $when->{not} ? !$_ : $_ } @ok
-               ;
-    
-    DEBUG_CONSTRAINTS && debug("'when' return value" => $return);
-    
+    DEBUG_CONSTRAINTS_WHEN && debug( "'when' value matches" => \@ok );
+
+    my $return
+        = $any
+        ? any { $when->{not} ? !$_ : $_ } @ok
+        : all { $when->{not} ? !$_ : $_ } @ok;
+
+    DEBUG_CONSTRAINTS_WHEN && debug( "'when' return value" => $return );
+
     return $return;
 }
 
+sub fetch_error_message {
+    my ( $self ) = @_;
+
+    my $error = HTML::FormFu::Exception::Constraint->new({
+        form      => $self->form,
+        parent    => $self->parent,
+        processor => $self,
+    });
+
+    return $error->message;
+}
+
 sub clone {
     my $self = shift;
 
@@ -339,7 +397,7 @@ See L<HTML::FormFu/"FORM LOGIC AND VALIDATION"> for further details.
 
 L<HTML::FormFu/constraints> can be called on any L<form|HTML::FormFu>,
 L<block element|HTML::FormFu::Element::Block> (includes fieldsets) or
-L<field element|HTML::FormFu::Element::_Field>.
+L<field element|HTML::FormFu::Role::Element::Field>.
 
 If called on a field element, no C<name> argument should be passed.
 
@@ -371,7 +429,7 @@ Argument: \@repeatable_count
 For constraints added to fields within a
 L<Repeatable|HTML::FormFu::Element::Repeatable> element, if C<only_on_reps>
 is set, the constraint will only be run for fields whose
-L<repeatable_count|HTML::FormFu::Element::_Field/repeatable_count>
+L<repeatable_count|HTML::FormFu::Role::Element::Field/repeatable_count>
 matches one of these set values.
 
 Not available for the constraints listed in
@@ -407,7 +465,7 @@ See L<HTML::FormFu/force_errors> for details.
 
 =head2 parent
 
-Returns the L<field|HTML::FormFu::Element::_Field> object that the constraint
+Returns the L<field|HTML::FormFu::Role::Element::Field> object that the constraint
 is associated with.
 
 =head2 form
@@ -471,8 +529,38 @@ to the callback sub. In this case all other keys are ignored, including not.
 You need to return a true value for the constraint to be applied or a false
 value to not apply it.
 
+The callback subroutine receives 2 arguments:
+
+=over 8
+
+=item 1
+
+C<$params> (hashref of all submitted parameters)
+
+=item 2
+
+C<$constraint> (the Constraint object)
+
 =back
 
+=back
+
+=head1 EXPERIMENTAL METHODS
+
+=head2 fetch_error_message
+
+Return value: $string
+
+Attempt to return the error message that would be used if this constraint 
+generated an error.
+
+This will generally be correct for simple constraints with a fixed message or
+which use a placeholder from a known value, such as 
+L<HTML::FormFu::Constraint::Min/min>.
+This will generally C<not> return the correct message for constraints which
+use L<HTML::FormFu::Role::Constraint::Others/others>, where the field with an
+error is not known without actually fully processing a form submission.
+
 =head1 CORE CONSTRAINTS
 
 =over
@@ -527,8 +615,6 @@ value to not apply it.
 
 =item L<HTML::FormFu::Constraint::Range>
 
-=item L<HTML::FormFu::Constraint::reCAPTCHA>
-
 =item L<HTML::FormFu::Constraint::Regex>
 
 =item L<HTML::FormFu::Constraint::Required>
@@ -541,6 +627,14 @@ value to not apply it.
 
 =back
 
+=head1 NON-CORE CONSTRAINTS AVAILABLE ON CPAN
+
+=over
+
+=item L<HTML::FormFu::Constraint::reCAPTCHA>
+
+=back
+
 =head1 CAVEATS
 
 See L<HTML::FormFu::Element::Repeatable/"Unsupported Constraints">
@@ -559,4 +653,4 @@ Sebastian Riedel, C<sri@oook.de>.
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
-=cut
\ No newline at end of file
+=cut
