@@ -1,7 +1,11 @@
 package HTML::FormFu::Constraint::Equal;
+{
+  $HTML::FormFu::Constraint::Equal::VERSION = '1.00';
+}
+use Moose;
+extends 'HTML::FormFu::Constraint';
 
-use strict;
-use base 'HTML::FormFu::Constraint::_others';
+with 'HTML::FormFu::Role::Constraint::Others';
 
 use HTML::FormFu::Util qw(
     DEBUG_CONSTRAINTS
@@ -38,13 +42,14 @@ sub process {
 
         if ( $self->not ) {
             if ( $value eq $EMPTY_STR ) {
+
                 # no error if both values are empty and not(1) is set
             }
             elsif ($ok) {
                 push @failed, $name;
             }
         }
-        elsif (!$ok) {
+        elsif ( !$ok ) {
             push @failed, $name;
         }
 
@@ -53,7 +58,7 @@ sub process {
 
     # special case for $self->not()
     # no errors if all values are empty
-    if ( $self->not
+    if (   $self->not
         && $value eq $EMPTY_STR
         && all { !defined || $_ eq $EMPTY_STR } values %values )
     {
@@ -106,6 +111,8 @@ sub _localize_args {
     return $self->parent->label;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -126,13 +133,13 @@ HTML::FormFu::Constraint::Equal - Multi-field Equality Constraint
 
 =head1 DESCRIPTION
 
-All fields named in L<HTML::FormFu::Constraint::_others/others> must have an equal value to the field this 
+All fields named in L<HTML::FormFu::Role::Constraint::Others/others> must have an equal value to the field this 
 constraint is attached to.
 
 =head1 SEE ALSO
 
 Is a sub-class of, and inherits methods from  
-L<HTML::FormFu::Constraint::_others>, L<HTML::FormFu::Constraint>
+L<HTML::FormFu::Role::Constraint::Others>, L<HTML::FormFu::Constraint>
 
 L<HTML::FormFu>
 
