@@ -1,7 +1,10 @@
 package HTML::FormFu::Constraint::MaxRange;
+{
+  $HTML::FormFu::Constraint::MaxRange::VERSION = '1.00';
+}
 
-use strict;
-use base 'HTML::FormFu::Constraint::Range';
+use Moose;
+extends 'HTML::FormFu::Constraint::Range';
 
 sub _localize_args {
     my ($self) = @_;
@@ -9,6 +12,8 @@ sub _localize_args {
     return $self->max;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -38,8 +43,8 @@ L</max> is an alias for L</maximum>.
 
 =head1 SEE ALSO
 
-Is a sub-class of, and inherits methods from L<HTML::FormFu::Range>, 
-L<HTML::FormFu::Constraint>
+Is a sub-class of, and inherits methods from
+L<HTML::FormFu::Constraint::Range>, L<HTML::FormFu::Constraint>
 
 L<HTML::FormFu>
 
@@ -51,3 +56,5 @@ Carl Franks C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
