@@ -1,7 +1,10 @@
 package HTML::FormFu::Constraint::Number;
+{
+  $HTML::FormFu::Constraint::Number::VERSION = '1.00';
+}
 
-use strict;
-use base 'HTML::FormFu::Constraint';
+use Moose;
+extends 'HTML::FormFu::Constraint';
 
 use Scalar::Util qw( looks_like_number );
 
@@ -15,6 +18,8 @@ sub constrain_value {
     return $self->not ? !$ok : $ok;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
