@@ -1,13 +1,17 @@
 package HTML::FormFu::Deflator::CompoundSplit;
+{
+  $HTML::FormFu::Deflator::CompoundSplit::VERSION = '1.00';
+}
 
 use Moose;
+use MooseX::Attribute::Chained;
 extends 'HTML::FormFu::Deflator';
 
 use HTML::FormFu::Constants qw( $EMPTY_STR $SPACE );
 
 has split       => ( is => 'rw', traits => ['Chained'] );
 has join        => ( is => 'rw', traits => ['Chained'] );
-has field_order => ( is => 'rw', traits  => ['Chained'] );
+has field_order => ( is => 'rw', traits => ['Chained'] );
 
 sub deflator {
     my ( $self, $value ) = @_;
@@ -102,7 +106,7 @@ Deflator to allow you to set several field's values at once.
 
 For use with a L<HTML::FormFu::Element::Multi> group of fields.
 
-A L<default|HTML::FormFu::Element::_Field/default> value passed to the 
+A L<default|HTML::FormFu::Role::Element::Field/default> value passed to the 
 L<Multi|HTML::FormFu::Element::Multi> field will be split according to the 
 L</split> setting, and its resulting parts passed to its child elements.
 
