@@ -1,25 +1,39 @@
 package HTML::FormFu::Element::Blank;
+{
+  $HTML::FormFu::Element::Blank::VERSION = '1.00';
+}
+use Moose;
 
-use strict;
-use base 'HTML::FormFu::Element::_Input';
+extends 'HTML::FormFu::Element';
+
+with 'HTML::FormFu::Role::Element::Input';
 
 use HTML::FormFu::Constants qw( $EMPTY_STR );
 
-sub label_tag {
-    return $EMPTY_STR;
-}
+after BUILD => sub {
+    my $self = shift;
+
+    $self->label_tag($EMPTY_STR);
+
+    #$self->field_type( $EMPTY_STR );
+    $self->render($EMPTY_STR);
+
+    return;
+};
 
 sub field_tag {
     return $EMPTY_STR;
 }
 
-sub render {
+override render => sub {
     return $EMPTY_STR;
-}
+};
 
-sub render_data_non_recursive {
+around render_data_non_recursive => sub {
     return;
-}
+};
+
+__PACKAGE__->meta->make_immutable;
 
 1;
 
@@ -32,8 +46,8 @@ HTML::FormFu::Element::Blank - blank element
 =head1 SEE ALSO
 
 Is a sub-class of, and inherits methods from 
-L<HTML::FormFu::Element::_Input>, 
-L<HTML::FormFu::Element::_Field>, 
+L<HTML::FormFu::Role::Element::Input>, 
+L<HTML::FormFu::Role::Element::Field>, 
 L<HTML::FormFu::Element>
 
 L<HTML::FormFu>
@@ -46,3 +60,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
