@@ -1,17 +1,23 @@
 package HTML::FormFu::Element::Button;
+{
+  $HTML::FormFu::Element::Button::VERSION = '1.00';
+}
+use Moose;
 
-use strict;
-use base 'HTML::FormFu::Element::_Input';
-use Class::C3;
+extends 'HTML::FormFu::Element';
 
-sub new {
-    my $self = shift->next::method(@_);
+with 'HTML::FormFu::Role::Element::Input';
+
+after BUILD => sub {
+    my ( $self, $args ) = @_;
 
     $self->field_type('button');
     $self->force_default(1);
 
-    return $self;
-}
+    return;
+};
+
+__PACKAGE__->meta->make_immutable;
 
 1;
 
@@ -36,8 +42,8 @@ L<HTML::FormFu::Element::Submit>
 =head1 SEE ALSO
 
 Is a sub-class of, and inherits methods from 
-L<HTML::FormFu::Element::_Input>, 
-L<HTML::FormFu::Element::_Field>, 
+L<HTML::FormFu::Role::Element::Input>, 
+L<HTML::FormFu::Role::Element::Field>, 
 L<HTML::FormFu::Element>
 
 L<HTML::FormFu>
@@ -50,3 +56,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
