@@ -1,10 +1,17 @@
 package HTML::FormFu::Element::Fieldset;
+{
+  $HTML::FormFu::Element::Fieldset::VERSION = '1.00';
+}
 
 use Moose;
 extends 'HTML::FormFu::Element::Block';
 
+use HTML::FormFu::Util qw( xml_escape );
+
 __PACKAGE__->mk_output_accessors(qw( legend ));
 
+__PACKAGE__->mk_attrs(qw( legend_attributes ));
+
 after BUILD => sub {
     my $self = shift;
 
@@ -17,7 +24,8 @@ sub render_data_non_recursive {
     my ( $self, $args ) = @_;
 
     my $render = $self->SUPER::render_data_non_recursive( {
-            legend => $self->legend,
+            legend            => $self->legend,
+            legend_attributes => xml_escape( $self->legend_attributes ),
             $args ? %$args : (),
         } );
 
@@ -44,6 +52,17 @@ Fieldset element.
 
 =head1 METHODS
 
+=head2 legend
+
+If L</legend> is set, it is used as the fieldset's legend
+
+=head2 legend_loc
+
+Arguments: $localization_key
+
+To set the legend to a localized string, set L</legend_loc> to a key in 
+your L10N file instead of using L</legend>.
+
 =head1 SEE ALSO
 
 Is a sub-class of, and inherits methods from 
