@@ -1,29 +1,31 @@
 package HTML::FormFu::Element::Number;
+{
+  $HTML::FormFu::Element::Number::VERSION = '1.00';
+}
 use Moose;
 
 extends 'HTML::FormFu::Element::Text';
 
 after BUILD => sub {
     my $self = shift;
-    
-    $self->field_type('number');
-    
-    $self->deflator( 'FormatNumber' );
-    $self->filter(   'FormatNumber' );
-    
+
+    $self->deflator('FormatNumber');
+    $self->filter('FormatNumber');
+
     return;
 };
 
 sub precision {
     my $self = shift;
-    
-    return $self->get_deflator({ type => 'FormatNumber' })->precision(@_);
+
+    return $self->get_deflator( { type => 'FormatNumber' } )->precision(@_);
 }
 
 sub trailing_zeroes {
     my $self = shift;
-    
-    return $self->get_deflator({ type => 'FormatNumber' })->trailing_zeroes(@_);
+
+    return $self->get_deflator( { type => 'FormatNumber' } )
+        ->trailing_zeroes(@_);
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -67,6 +69,14 @@ Set the precision for the number. Defaults to C<2>.
 
 If this is set to C<1> the number has trailing zeroes. Defaults to C<0>. 
 
+=head2 CHANGED BEHAVIOUR AS OF VERSION 0.09011
+
+Previous to version 0.09011, this element generated an input field with
+C<type="number"> attribute which was not valid xhtml.
+Neither was the formatted numbers valid html5 - which expects the value to
+be a floating-point number.
+This element now generates a C<type="text"> attribute.
+
 =head1 SEE ALSO
 
 L<HTML::FormFu::Deflator::FormatNumber>
