@@ -1,33 +1,45 @@
 package HTML::FormFu::Element::Repeatable;
+{
+  $HTML::FormFu::Element::Repeatable::VERSION = '1.00';
+}
 
-use strict;
-use base 'HTML::FormFu::Element::Block';
+use Moose;
+use MooseX::Attribute::Chained;
+extends 'HTML::FormFu::Element::Block';
 
 use HTML::FormFu::Util qw( DEBUG_PROCESS debug );
-use mro 'c3';
 use List::Util qw( first );
 use Carp qw( croak );
 
-__PACKAGE__->mk_item_accessors( qw(
-        _original_elements
-        increment_field_names
-        counter_name
-        repeatable_delimiter
-) );
+has counter_name => ( is => 'rw', traits => ['Chained'] );
 
-sub new {
-    my $self = shift->next::method(@_);
+has _original_elements => ( is => 'rw' );
+
+has increment_field_names => (
+    is      => 'rw',
+    default => 1,
+    lazy    => 1,
+    traits  => ['Chained'],
+);
+
+# This attribute is currently not documented as FF::Model::HashRef
+# only supports '_'
+
+has repeatable_delimiter => (
+    is      => 'rw',
+    default => '_',
+    lazy    => 1,
+    traits  => ['Chained'],
+);
+
+after BUILD => sub {
+    my $self = shift;
 
     $self->filename('repeatable');
     $self->is_repeatable(1);
-    $self->increment_field_names(1);
-# TODO
-    # This setter is currently not documentes as FF::Model::HashRef
-    # only supports '_'
-    $self->repeatable_delimiter('_');
 
-    return $self;
-}
+    return;
+};
 
 sub repeat {
     my ( $self, $count ) = @_;
@@ -63,11 +75,11 @@ sub repeat {
     # If nested_name is *not* set, we add the repeatable counter to the names
     # of the child elements (leaves of the element tree).
     my $nested_name = $self->nested_name;
-    if (defined $nested_name && length $nested_name) {
-        return $self->_repeat_containing_block( $count );
+    if ( defined $nested_name && length $nested_name ) {
+        return $self->_repeat_containing_block($count);
     }
     else {
-        return $self->_repeat_child_elements( $count );
+        return $self->_repeat_child_elements($count);
     }
 }
 
@@ -80,7 +92,7 @@ sub _repeat_containing_block {
     # nested_name attribute of the Repeatable element, thus we extended
     # FF::Elements::_Field nested_names method to ignore Repeatable elements.
     my $nested_name = $self->nested_name;
-    $self->original_nested_name( $nested_name );
+    $self->original_nested_name($nested_name);
 
     # delimiter between nested_name and the incremented counter
     my $delimiter = $self->repeatable_delimiter;
@@ -88,6 +100,7 @@ sub _repeat_containing_block {
     my @return;
 
     for my $rep ( 1 .. $count ) {
+
         # create clones of elements and put them in a new block
         my @clones = map { $_->clone } @$children;
         my $block = $self->element('Block');
@@ -100,6 +113,7 @@ sub _repeat_containing_block {
         $block->repeatable_count($rep);
 
         if ( $self->increment_field_names ) {
+
             # store the original nested_name attribute for later usage when
             # building the original nested name
             $block->original_nested_name( $block->nested_name )
@@ -111,6 +125,7 @@ sub _repeat_containing_block {
             for my $field ( @{ $block->get_fields } ) {
 
                 if ( defined( my $name = $field->name ) ) {
+
                     # store original name for later usage when
                     # replacing the field names in constraints
                     $field->original_name($name)
@@ -118,7 +133,8 @@ sub _repeat_containing_block {
 
                     # store original nested name for later usage when
                     # replacing the field names in constraints
-                    $field->original_nested_name( $field->build_original_nested_name )
+                    $field->original_nested_name(
+                        $field->build_original_nested_name )
                         if !defined $field->original_nested_name;
                 }
             }
@@ -126,7 +142,9 @@ sub _repeat_containing_block {
 
         _reparent_children($block);
 
-        for my $field ( @{ $block->get_fields } ) {
+        my @fields = @{ $block->get_fields };
+
+        for my $field (@fields) {
             map { $_->parent($field) }
                 @{ $field->_deflators },
                 @{ $field->_filters },
@@ -138,60 +156,29 @@ sub _repeat_containing_block {
                 ;
         }
 
-        my $block_fields = $block->get_fields;
-
-        my @block_constraints = map { @{ $_->get_constraints } } @$block_fields;
-
-        # rename any 'others' fields
-        my @others_constraints = grep { defined $_->others }
-            grep { $_->can('others') } @block_constraints;
-
-        for my $constraint (@others_constraints) {
-            my $others = $constraint->others;
-            if ( !ref $others ) {
-                $others = [$others];
-            }
-            my @new_others;
-
-            for my $name (@$others) {
-                my $field
-                    = ( first { $_->original_nested_name eq $name }
-                    @$block_fields )
-                    || first { $_->original_name eq $name } @$block_fields;
-
-                if ( defined $field ) {
-                    push @new_others, $field->nested_name;
-                }
-                else {
-                    push @new_others, $name;
-                }
-            }
-
-            $constraint->others( \@new_others );
-        }
-
-        # rename any 'when' fields
-        my @when_constraints = grep { defined $_->when } @block_constraints;
-
-        for my $constraint (@when_constraints) {
-            my $when = $constraint->when;
-            my $name = $when->{field};
-
-            my $field
-                = first { $_->original_nested_name eq $name } @$block_fields;
-
-            if ( defined $field ) {
-                $when->{field} = $field->nested_name;
-            }
+        for my $field (@fields) {
+            map { $_->repeatable_repeat( $self, $block ) }
+                @{ $field->_constraints };
         }
 
         push @return, $block;
-
     }
 
     return \@return;
 }
 
+sub get_field_with_original_name {
+    my ( $self, $name, $fields ) = @_;
+
+    my $field = first { $_->original_nested_name eq $name }
+    grep { defined $_->original_nested_name } @$fields;
+
+    $field ||= first { $_->original_name eq $name }
+    grep { defined $_->original_name } @$fields;
+
+    return $field;
+}
+
 sub _repeat_child_elements {
     my ( $self, $count ) = @_;
 
@@ -222,14 +209,16 @@ sub _repeat_child_elements {
                     $field->original_nested_name( $field->nested_name )
                         if !defined $field->original_nested_name;
 
-                    $field->name(${name} . $delimiter . $rep);
+                    $field->name( ${name} . $delimiter . $rep );
                 }
             }
         }
 
         _reparent_children($block);
 
-        for my $field ( @{ $block->get_fields } ) {
+        my @fields = @{ $block->get_fields };
+
+        for my $field (@fields) {
             map { $_->parent($field) }
                 @{ $field->_deflators },
                 @{ $field->_filters },
@@ -241,55 +230,12 @@ sub _repeat_child_elements {
                 ;
         }
 
-        my $block_fields = $block->get_fields;
-
-        my @block_constraints = map { @{ $_->get_constraints } } @$block_fields;
-
-        # rename any 'others' fields
-        my @others_constraints = grep { defined $_->others }
-            grep { $_->can('others') } @block_constraints;
-
-        for my $constraint (@others_constraints) {
-            my $others = $constraint->others;
-            if ( !ref $others ) {
-                $others = [$others];
-            }
-            my @new_others;
-
-            for my $name (@$others) {
-                my $field
-                    = ( first { $_->original_nested_name eq $name }
-                    @$block_fields )
-                    || first { $_->original_name eq $name } @$block_fields;
-
-                if ( defined $field ) {
-                    push @new_others, $field->nested_name;
-                }
-                else {
-                    push @new_others, $name;
-                }
-            }
-
-            $constraint->others( \@new_others );
-        }
-
-        # rename any 'when' fields
-        my @when_constraints = grep { defined $_->when } @block_constraints;
-
-        for my $constraint (@when_constraints) {
-            my $when = $constraint->when;
-            my $name = $when->{field};
-
-            my $field
-                = first { $_->original_nested_name eq $name } @$block_fields;
-
-            if ( defined $field ) {
-                $when->{field} = $field->nested_name;
-            }
+        for my $field (@fields) {
+            map { $_->repeatable_repeat( $self, $block ) }
+                @{ $field->_constraints };
         }
 
         push @return, $block;
-
     }
 
     return \@return;
@@ -315,11 +261,13 @@ sub process {
     my $count        = 1;
 
     if ( defined $counter_name && defined $form->query ) {
+
         # are we in a nested-repeatable?
         my $parent = $self;
 
         while ( defined( $parent = $parent->parent ) ) {
-            my $field = $parent->get_field({ original_name => $counter_name });
+            my $field
+                = $parent->get_field( { original_name => $counter_name } );
 
             if ( defined $field ) {
                 $counter_name = $field->nested_name;
@@ -327,7 +275,7 @@ sub process {
             }
         }
 
-        my $input = $form->query->param( $counter_name );
+        my $input = $form->query->param($counter_name);
 
         if ( defined $input && $input =~ /^[1-9][0-9]*\z/ ) {
             $count = $input;
@@ -335,12 +283,12 @@ sub process {
     }
 
     if ( !$self->_original_elements ) {
-        DEBUG_PROCESS && debug("calling \$repeatable->count($count)");
+        DEBUG_PROCESS && debug("calling \$repeatable->repeat($count)");
 
         $self->repeat($count);
     }
 
-    return $self->next::method(@_);
+    return $self->SUPER::process(@_);
 }
 
 sub content {
@@ -371,6 +319,8 @@ sub string {
     return $html;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -435,7 +385,7 @@ inherits the Repeatable's display settings, such as L</attributes> and
 L</tag>.
 
 For all constraints attached to fields within a Repeatable block which use
-either L<others|HTML::FormFu::Constraint::_others/others> or
+either L<others|HTML::FormFu::Role::Constraint::Others/others> or
 L<when|HTML::FormFu::Constraint/when> containing names of fields within
 the same Repeatable block, when L<repeat> is called, those names will
 automatically be updated to the new nested-name for each field (taking
@@ -502,6 +452,10 @@ used to determine whether any element is a child of a Repeatable element.
 
 Only available after L<repeat> has been called.
 
+=head2 repeatable_count_no_inherit
+
+A non-inheriting variant of L</repeatable_count>.
+
 =head2 nested_name
 
 If the L</nested_name> attribute is set, the naming scheme of the Repeatable
@@ -613,7 +567,7 @@ a setter.
 
 =head2 Unsupported Constraints
 
-Note that constraints with an L<others|HTML::FormFu::Constraint::_others> 
+Note that constraints with an L<others|HTML::FormFu::Role::Constraint::Others> 
 method do not work correctly within a Repeatable block. Currently, these are:
 L<AllOrNone|HTML::FormFu::Constraint::AllOrNone>, 
 L<DependOn|HTML::FormFu::Constraint::DependOn>, 
