@@ -1,16 +1,20 @@
 package HTML::FormFu::Element::Reset;
+{
+  $HTML::FormFu::Element::Reset::VERSION = '1.00';
+}
+use Moose;
 
-use strict;
-use base 'HTML::FormFu::Element::Button';
-use mro 'c3';
+extends 'HTML::FormFu::Element::Button';
 
-sub new {
-    my $self = shift->next::method(@_);
+after BUILD => sub {
+    my $self = shift;
 
     $self->field_type('reset');
 
-    return $self;
-}
+    return;
+};
+
+__PACKAGE__->meta->make_immutable;
 
 1;
 
@@ -34,8 +38,8 @@ Reset button form field.
 
 Is a sub-class of, and inherits methods from 
 L<HTML::FormFu::Element::Button>, 
-L<HTML::FormFu::Element::_Input>, 
-L<HTML::FormFu::Element::_Field>, 
+L<HTML::FormFu::Role::Element::Input>, 
+L<HTML::FormFu::Role::Element::Field>, 
 L<HTML::FormFu::Element>
 
 L<HTML::FormFu>
