@@ -1,16 +1,20 @@
 package HTML::FormFu::Element::Src;
+{
+  $HTML::FormFu::Element::Src::VERSION = '1.00';
+}
 
-use strict;
-use base 'HTML::FormFu::Element::Block';
-use mro 'c3';
+use Moose;
+extends 'HTML::FormFu::Element::Block';
 
-sub new {
-    my $self = shift->next::method(@_);
+after BUILD => sub {
+    my $self = shift;
 
     $self->tag(undef);
 
-    return $self;
-}
+    return;
+};
+
+__PACKAGE__->meta->make_immutable;
 
 1;
 
