@@ -1,8 +1,11 @@
 package HTML::FormFu::Inflator::CompoundDateTime;
+{
+  $HTML::FormFu::Inflator::CompoundDateTime::VERSION = '1.00';
+}
 
-use strict;
-use base 'HTML::FormFu::Inflator';
-use Class::C3;
+use Moose;
+use MooseX::Attribute::Chained;
+extends 'HTML::FormFu::Inflator';
 
 use HTML::FormFu::Constants qw( $EMPTY_STR );
 use DateTime;
@@ -11,9 +14,8 @@ use List::MoreUtils qw( none );
 use Scalar::Util qw( reftype );
 use Carp qw( croak );
 
-__PACKAGE__->mk_item_accessors(qw( strptime ));
-
-__PACKAGE__->mk_accessors(qw( field_order ));
+has strptime    => ( is => 'rw', traits => ['Chained'] );
+has field_order => ( is => 'rw', traits => ['Chained'] );
 
 my @known_fields = qw( year month day hour minute second nanosecond time_zone );
 
@@ -55,7 +57,7 @@ sub inflator {
         my $strptime = $self->strptime;
         my %args;
 
-        if ( ( reftype( $strptime ) || '' ) eq 'HASH' ) {
+        if ( ( reftype($strptime) || '' ) eq 'HASH' ) {
             %args = %$strptime;
         }
         else {
@@ -70,6 +72,8 @@ sub inflator {
     return $dt;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
