@@ -1,15 +1,17 @@
 package HTML::FormFu::OutputProcessor;
+{
+  $HTML::FormFu::OutputProcessor::VERSION = '1.00';
+}
 use Moose;
 use MooseX::Attribute::Chained;
 
-with 'HTML::FormFu::Role::HasParent',
-     'HTML::FormFu::Role::Populate';
+with 'HTML::FormFu::Role::HasParent', 'HTML::FormFu::Role::Populate';
 
 use HTML::FormFu::ObjectUtil qw( form parent );
 use Scalar::Util qw( reftype );
 use Carp qw( croak );
 
-has type => ( is => 'rw', traits  => ['Chained'] );
+has type => ( is => 'rw', traits => ['Chained'] );
 
 sub clone {
     my ($self) = @_;
