@@ -1,8 +1,10 @@
 package HTML::FormFu::Plugin::StashValid;
+{
+  $HTML::FormFu::Plugin::StashValid::VERSION = '1.00';
+}
 
-use strict;
-use base 'HTML::FormFu::Plugin';
-use mro 'c3';
+use Moose;
+extends 'HTML::FormFu::Plugin';
 
 sub post_process {
     my ($self) = @_;
@@ -17,6 +19,8 @@ sub post_process {
     return;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
