@@ -1,9 +1,13 @@
 package HTML::FormFu::Validator::Callback;
+{
+  $HTML::FormFu::Validator::Callback::VERSION = '1.00';
+}
 
-use strict;
-use base 'HTML::FormFu::Validator';
+use Moose;
+use MooseX::Attribute::Chained;
+extends 'HTML::FormFu::Validator';
 
-__PACKAGE__->mk_item_accessors(qw( callback ));
+has callback => ( is => 'rw', traits => ['Chained'] );
 
 sub validate_value {
     my ( $self, $value ) = @_;
@@ -17,6 +21,8 @@ sub validate_value {
     return $ok;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -63,3 +69,5 @@ Carl Franks C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
